@@ -72,13 +72,14 @@ class StreamChannel(BaseChannel):
 
     # Override
     def _set_socket(self, sock: Optional[socket.socket]):
-        # 1. check old socket
-        old = self._get_socket()
+        # 1. replace old socket
+        old = self.__sock
+        self.__sock = sock
+        self._refresh_flags(sock=sock)
+        # 2. close old socket
         if old is not None and old is not sock:
             if is_opened(sock=old):
                 close_socket(sock=old)
-        # 2. set new socket
-        self.__sock = sock
 
     # Override
     def _create_reader(self):
@@ -93,5 +94,5 @@ def close_socket(sock: socket.socket):
     try:
         # sock.shutdown(socket.SHUT_RDWR)
         sock.close()
-    except socket.error:
-        pass
+    except socket.error as error:
+        print('[TCP] failed to close socket: %s, %s' % (error, sock))
