@@ -30,12 +30,11 @@
 
 import socket
 from abc import ABC
-from threading import Thread
-from typing import Optional, Set
+from typing import Optional, Iterable
 
 from startrek.types import Address, AddressPairMap
-from startrek.fsm import Runnable
-from startrek.net.channel import is_opened
+from startrek.fsm import Runnable, Daemon
+from startrek.net.channel import is_opened, get_local_address
 from startrek import Channel
 from startrek import Connection, ConnectionDelegate
 from startrek import BaseConnection, ActiveConnection
@@ -68,12 +67,17 @@ class StreamHub(BaseHub, ABC):
         super().__init__(delegate=delegate)
         self.__channel_pool = ChannelPool()
 
-    def put_channel(self, channel: Channel):
-        """ cache channel """
-        self.__channel_pool.set(remote=channel.remote_address, local=channel.local_address, item=channel)
+    #
+    #   Channel
+    #
+
+    # noinspection PyMethodMayBeStatic
+    def _create_channel(self, sock: socket.socket, remote: Optional[Address], local: Optional[Address]) -> Channel:
+        """ create channel with socket & addresses """
+        return StreamChannel(remote=remote, local=local, sock=sock)
 
     # Override
-    def _all_channels(self) -> Set[Channel]:
+    def _all_channels(self) -> Iterable[Channel]:
         """ get a copy of all channels """
         return self.__channel_pool.items
 
@@ -82,27 +86,30 @@ class StreamHub(BaseHub, ABC):
         """ remove cached channel """
         self.__channel_pool.remove(remote=channel.remote_address, local=channel.local_address, item=channel)
 
+    def _get_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
+        """ get cached channel """
+        return self.__channel_pool.get(remote=remote, local=local)
+
+    def _set_channel(self, channel: Channel):
+        """ cache channel """
+        self.__channel_pool.set(remote=channel.remote_address, local=channel.local_address, item=channel)
+
     # Override
     def open(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
         assert remote is not None, 'remote address empty: %s, %s' % (remote, local)
         return self.__channel_pool.get(remote=remote, local=None)
 
-    # Override
-    def _get_connection(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Connection]:
-        return super()._get_connection(remote=remote, local=None)
-
 
 class ServerHub(StreamHub, Runnable):
     """ Stream Server Hub """
 
-    def __init__(self, delegate: ConnectionDelegate, daemon: bool = False):
+    def __init__(self, delegate: ConnectionDelegate, daemonic: bool = True):
         super().__init__(delegate=delegate)
         self.__local = None   # Address
         self.__master = None  # socket.socket
         # running thread
-        self.__thread: Optional[Thread] = None
+        self.__daemon = Daemon(target=self.run, daemonic=daemonic)
         self.__running = False
-        self.__daemon = daemon
 
     def _get_master(self) -> Optional[socket.socket]:
         return self.__master
@@ -137,7 +144,11 @@ class ServerHub(StreamHub, Runnable):
 
     def bind(self, address: Optional[Address] = None, host: Optional[str] = None, port: Optional[int] = 0):
         if address is None:
-            address = (host, port)
+            if port > 0:
+                address = (host, port)
+            else:
+                address = self.__local
+                assert address is not None, 'local address not set'
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
         sock.setblocking(True)
@@ -145,26 +156,16 @@ class ServerHub(StreamHub, Runnable):
         sock.listen(1)
         # sock.setblocking(False)
         self._set_master(master=sock)
-        self.local_address = address
+        self.__local = address
 
     def start(self):
-        self.__force_stop()
+        self.stop()
         self.__running = True
-        thr = Thread(target=self.run, daemon=self.__daemon)
-        self.__thread = thr
-        thr.start()
-        return thr
-
-    def __force_stop(self):
-        self.__running = False
-        t: Thread = self.__thread
-        if t is not None:
-            # waiting 2 seconds for stopping the thread
-            self.__thread = None
-            t.join(timeout=2.0)
+        return self.__daemon.start()
 
     def stop(self):
-        self.__force_stop()
+        self.__running = False
+        self.__daemon.stop()
 
     # Override
     def run(self):
@@ -182,8 +183,9 @@ class ServerHub(StreamHub, Runnable):
     def _accept(self, sock: socket.socket, remote: Address, local: Address):
         # override for user-customized channel
         assert sock is not None, 'socket error: %s, %s' % (remote, local)
-        channel = StreamChannel(remote=remote, local=None, sock=sock)
-        self.put_channel(channel=channel)
+        channel = self._create_channel(sock=sock, remote=remote, local=local)
+        assert channel is not None, 'failed to create socket channel: %s, remote=%s, local=%s' % (sock, remote, local)
+        self._set_channel(channel=channel)
 
 
 class ClientHub(StreamHub):
@@ -200,19 +202,20 @@ class ClientHub(StreamHub):
     def open(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
         channel = super().open(remote=remote, local=local)
         if channel is None:  # and remote is not None:
-            channel = self._create_channel(remote=remote, local=local)
+            channel = self.__create_socket_channel(remote=remote, local=local)
             if channel is not None:
-                self.put_channel(channel=channel)
+                self._set_channel(channel=channel)
         return channel
 
-    # noinspection PyMethodMayBeStatic
-    def _create_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
+    def __create_socket_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
         # override for user-customized channel
         try:
             sock = create_socket(remote=remote, local=local)
-            return StreamChannel(remote=remote, local=None, sock=sock)
+            if local is None:
+                local = get_local_address(sock=sock)
+            return self._create_channel(sock=sock, remote=remote, local=local)
         except socket.error as error:
-            print('[TCP] creating connection %s -> %s error: %s' % (local, remote, error))
+            print('[TCP] failed to create channel %s -> %s: %s' % (local, remote, error))
 
 
 def create_socket(remote: Address, local: Optional[Address]) -> Optional[socket.socket]:
